module MediServ::API
  require_relative "format"

  class Encoder
    def encode(h)
      build_tree(Node.new, h).to_ini
    end

    def build_tree(node, h)
      h.each do |k, v|
        case v
        when Hash
          child = Node.new(name: [node.name, k].compact.join('.'))
          node.children << child
          build_tree(child, v)
        else
          node.attributes[k] = v
        end
      end
      node
    end
  end

  class Node
    include Format

    attr_accessor :name, :attributes, :children

    def initialize(name: nil, attributes: {}, children: [])
      @name = name
      @attributes = attributes
      @children = children
    end

    def to_ini(io = StringIO.new)
      if name
        io << '['
        io << name
        io << ']'
        io << "\n"
      end
      attributes.each do |k, v|
        next unless k && v
        io << k.to_s
        io << '='

        v = case v
        when Float
          io << fmt_amount(v)
        when String
          io << v.tr("\n", '')
        else
          io << v
        end
        io << "\n"
      end
      children.each do |c|
        io << "\n"
        c.to_ini(io)
      end
      io.string.encode('windows-1252')
    end
  end
end
